Init()
{
	// League name header (is defined in each mode file)
	if (!isdefined(game["rules_leagueString"]))
		game["rules_leagueString"] = &"^1Unknown pam mode name";
	if (!isdefined(game["rules_formatString"]))
		game["rules_formatString"] = &"";
	if (!isdefined(game["rules_stringColor"]))
		game["rules_stringColor"] = (1, 1, 0);
	if (!isDefined(game["ruleCvars"]))
		game["ruleCvars"] = [];




	// Rules load
	loaded = false;
	switch (level.gametype)
	{
		case "sd": 	loaded = Load_SD_Rules(); break;
		case "dm": 	loaded = Load_DM_Rules(); break;
		case "tdm": 	loaded = Load_TDM_Rules(); break;
		case "ctf": 	loaded = Load_CTF_Rules(); break;
		case "strat": 	loaded = Load_Strat_Rules(); break;
	}
	assert(loaded);

	/#
	if (game["ruleCvars"].size == 0 && !game["is_public_mode"])
		assertMsg("Rule cvars was not loaded!");
	#/


	// Precache
	if (game["firstInit"])
	{
		precacheString(game["rules_leagueString"]);
		precacheString(game["rules_formatString"]);

		assert(level.pam_mode != "");

		_2v2 = PAMModeContains("2v2");
		russian = PAMModeContains("russian");
		lan = PAMModeContains("lan");

		game["leagueOptionsString"] = &"";
		if (_2v2 && russian && lan)	game["leagueOptionsString"] = &"2v2 | Russian | LAN";
		if (!_2v2 && russian && lan)	game["leagueOptionsString"] = &"Russian | LAN";
		if (_2v2 && !russian && lan)	game["leagueOptionsString"] = &"2v2 | LAN";
		if (_2v2 && russian && !lan)	game["leagueOptionsString"] = &"2v2 | Russian";
		if (_2v2 && !russian && !lan)	game["leagueOptionsString"] = &"2v2";
		if (!_2v2 && russian && !lan)	game["leagueOptionsString"] = &"Russian";
		if (!_2v2 && !russian && lan)	game["leagueOptionsString"] = &"LAN";

		precacheString(game["leagueOptionsString"]);
	}

	println("Loaded rules for pam mode: " + level.pam_mode);
}

ruleCvarDefault(array, cvarName, value)
{
	cvarName = toLower(cvarName);
	array[cvarName] = value;
	return array;
}


IsValidPAMMode(cvar, value_now, registerTime)
{
	isValid = IsValidPAMModeForGametype(level.gametype, value_now);

	if (level.gametype == "strat" && !registerTime)
		isValid = false;

	if (!isValid)
	{
		// Print not valid warning
		if (!registerTime)
		{
			iprintln("^3DVAR change detected: ^2" + value_now + "^3 is not a valid value for dvar ^2" + cvar["name"]);

			if (level.gametype == "sd")
			{
				iprintln("^3Following values are valid:");
				iprintln("^7pub, pcw, comp");
				iprintln("^7mr3, mr10, mr12, mr15, 20rounds");
				iprintln("^72v2, russian, lan");
			}
			else if (level.gametype == "dm")
			{
				iprintln("^3Following values are valid:");
				iprintln("^7pub, pcw, warmup");
				iprintln("^715min, 30min, 60min, unlim");
				iprintln("^72v2, russian, lan");
			}
			else if (level.gametype == "tdm")
			{
				iprintln("^3Following values are valid:");
				iprintln("^7pub, pcw");
				iprintln("^715min, 30min, 60min, unlim");
				iprintln("^72v2, russian, lan");
			}
			else if (level.gametype == "ctf")
			{
				iprintln("^3Following values are valid:");
				iprintln("^7pub, pcw");
				iprintln("^715min, 30min, 60min, unlim");
				iprintln("^72v2, russian, lan");
			}
			else
				iprintln("^3Change to other gametype!");

		}
	}

	if (!isValid && registerTime)
	{
		println("Cvar " + cvar["name"] + " has invalid value '"+value_now+"'. Using default value '" + cvar["defaultValue"] + "'.");
	}

	return isValid;
}


IsValidPAMModeForGametype(gametype, pammode)
{
	isValid = true;

	array = maps\mp\gametypes\global\string::splitString(pammode, "_");

	if (array.size <= 0)
		isValid = false;

	// Submodes
	for (i = 0; i < array.size; i++)
	{
		if (i == 0)
		{
			// First must be league
			if ((gametype == "sd"  && array[0] != "pub" && array[0] != "pcw" && array[0] != "comp") ||
			    (gametype == "dm"  && array[0] != "pub" && array[0] != "pcw" && array[0] != "warmup") ||
			    (gametype == "tdm" && array[0] != "pub" && array[0] != "pcw") ||
			    (gametype == "ctf" && array[0] != "pub" && array[0] != "pcw"))
			{
				isValid = false;
				break;
			}
		}
		else
		{
			if (!IsToggleSubPamMode(array[i]) && (
			    (gametype == "sd"  && array[i] != "mr3" && array[i] != "mr10" && array[i] != "mr12" && array[i] != "mr15" && array[i] != "20rounds") ||
			    (gametype == "dm"  && array[i] != "15min" && array[i] != "30min" && array[i] != "60min" && array[i] != "unlim") ||
			    (gametype == "tdm" && array[i] != "15min" && array[i] != "30min" && array[i] != "60min" && array[i] != "unlim") ||
			    (gametype == "ctf" && array[i] != "15min" && array[i] != "30min" && array[i] != "60min" && array[i] != "unlim")))
			{
				isValid = false;
				break;
			}
		}
	}
	return isValid;
}


PAMModeContains(string)
{
	array = maps\mp\gametypes\global\string::splitString(level.pam_mode, "_");

	for (i = 0; i < array.size; i++)
	{
		if (array[i] == string)
		{
			return true;
		}
	}
	return false;
}



IsToggleSubPamMode(name)
{
	switch(name)
	{
		case "2v2":
		case "russian":
		case "lan":
			return true;
	}
	return false;
}

LoadToggleSubPamMode(name)
{
	switch(name)
	{
		case "2v2": 	maps\mp\gametypes\rules\_2v2::Load(); return true;
		case "russian": maps\mp\gametypes\rules\russian::Load(); return true;
		case "lan": 	maps\mp\gametypes\rules\lan::Load(); return true;
	}
	return false;
}

Load_SD_Rules()
{
	if (level.pam_mode == "pub")
	{
		maps\mp\gametypes\rules\sd\pub::Load();
		return true;
	}

	array = maps\mp\gametypes\global\string::splitString(level.pam_mode, "_");

	if (array.size <= 0)
		return false;

	// First must be league
	if (array[0] == "pcw")
		maps\mp\gametypes\rules\sd\pcw::Load();
	else if (array[0] == "comp")
		maps\mp\gametypes\rules\sd\comp::Load();
	else
		return false;

	// Score settings
	for (i = 1; i < array.size; i++)
	{
		switch(array[i])
		{
			case "mr3": 	maps\mp\gametypes\rules\sd\score\mr3::Load(); break;
			case "mr10": 	maps\mp\gametypes\rules\sd\score\mr10::Load(); break;
			case "mr12": 	maps\mp\gametypes\rules\sd\score\mr12::Load(); break;
			case "mr15": 	maps\mp\gametypes\rules\sd\score\mr15::Load(); break;
			case "20rounds": maps\mp\gametypes\rules\sd\score\_20rounds::Load(); break;
			default:
				valid = LoadToggleSubPamMode(array[i]);
				if (!valid) return false;
				break;
		}
	}

	return true;
}

Load_DM_Rules()
{
	if (level.pam_mode == "pub")
	{
		maps\mp\gametypes\rules\dm\pub::Load();
		return true;
	}

	array = maps\mp\gametypes\global\string::splitString(level.pam_mode, "_");

	if (array.size <= 0)
		return false;

	// First must be league
	if (array[0] == "pcw")
		maps\mp\gametypes\rules\dm\pcw::Load();
	else if (array[0] == "warmup")
		maps\mp\gametypes\rules\dm\warmup::Load();
	else
		return false;

	// Score settings
	for (i = 1; i < array.size; i++)
	{
		switch(array[i])
		{
			case "15min": 	maps\mp\gametypes\rules\dm\score\_15min::Load(); break;
			case "30min": 	maps\mp\gametypes\rules\dm\score\_30min::Load(); break;
			case "60min": 	maps\mp\gametypes\rules\dm\score\_60min::Load(); break;
			case "unlim": 	maps\mp\gametypes\rules\dm\score\unlim::Load(); break;
			default:
				valid = LoadToggleSubPamMode(array[i]);
				if (!valid) return false;
				break;
		}
	}

	return true;
}

Load_TDM_Rules()
{
	if (level.pam_mode == "pub")
	{
		maps\mp\gametypes\rules\tdm\pub::Load();
		return true;
	}

	array = maps\mp\gametypes\global\string::splitString(level.pam_mode, "_");

	if (array.size <= 0)
		return false;

	// First must be league
	if (array[0] == "pcw")
		maps\mp\gametypes\rules\tdm\pcw::Load();
	else
		return false;

	// Score settings
	for (i = 1; i < array.size; i++)
	{
		switch(array[i])
		{
			case "15min": 	maps\mp\gametypes\rules\tdm\score\_15min::Load(); break;
			case "30min": 	maps\mp\gametypes\rules\tdm\score\_30min::Load(); break;
			case "60min": 	maps\mp\gametypes\rules\tdm\score\_60min::Load(); break;
			case "unlim": 	maps\mp\gametypes\rules\tdm\score\unlim::Load(); break;
			default:
				valid = LoadToggleSubPamMode(array[i]);
				if (!valid) return false;
				break;
		}
	}

	return true;
}

Load_CTF_Rules()
{
	if (level.pam_mode == "pub")
	{
		maps\mp\gametypes\rules\ctf\pub::Load();
		return true;
	}

	array = maps\mp\gametypes\global\string::splitString(level.pam_mode, "_");

	if (array.size <= 0)
		return false;

	// First must be league
	if (array[0] == "pcw")
		maps\mp\gametypes\rules\ctf\pcw::Load();
	else
		return false;

	// Score settings
	for (i = 1; i < array.size; i++)
	{
		switch(array[i])
		{
			case "15min": 	maps\mp\gametypes\rules\ctf\score\_15min::Load(); break;
			case "30min": 	maps\mp\gametypes\rules\ctf\score\_30min::Load(); break;
			case "60min": 	maps\mp\gametypes\rules\ctf\score\_60min::Load(); break;
			case "unlim": 	maps\mp\gametypes\rules\ctf\score\unlim::Load(); break;
			default:
				valid = LoadToggleSubPamMode(array[i]);
				if (!valid) return false;
				break;
		}
	}

	return true;
}

Load_Strat_Rules()
{
    maps\mp\gametypes\rules\strat\strat::Load();

    return true;
}
